//+------------------------------------------------------------------+
//|                                                    SignalSOM.mqh |
//|                                                        2011, ssn |
//|                                              hornsVIEW@gmail.com |
//+------------------------------------------------------------------+
#property copyright "2011 ssn"
#property link      "hornsVIEW@gmail.com"
//+------------------------------------------------------------------+
//| include files                                                    |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of a custom Self Organizing Map algorithm.         |
//| Type=SignalAdvanced                                              |
//| Name=SOM                                                         |
//| Class=CSignalSOM                                                 |
//| Page=SignalSOM                                                   |
//| Parameter=XMin,int,5,X long min.                                 |
//| Parameter=XMax,int,8,X long max.                                 |
//| Parameter=YMin,int,5,Y long min.                                 |
//| Parameter=YMax,int,8,Y long max.                                 |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
#define GRID 144
#define INPUT 5
static string GridWeights[GRID][GRID];

//+------------------------------------------------------------------+
//| CSignalSOM class                                                 |
//| Purpose: A class of a module of trade signals,                   |
//| on, ssn's simple entry algorithm                                 |
//+------------------------------------------------------------------+
class CSignalSOM : public CExpertSignal
  {
protected:
   
   //--- adjusted parameters
   int               m_x_min;      
   int               m_x_max;  
   int               m_y_min;      
   int               m_y_max;  
   
public:
   virtual int       ShortCondition();      //checks conditions for sell
   virtual int       LongCondition();       //checks conditions for buy
   
   //--- method of verification of settings
   virtual bool      ValidationSettings();
   //---
   virtual double    O(int ind)         { return(Open(ind)-Open(ind+1));  }
   virtual double    H(int ind)         { return(High(ind)-High(ind+1));  }
   virtual double    V(int ind)         { return(High(ind)-Low(ind));     }
   virtual double    L(int ind)         { return(Low(ind)-Low(ind+1));    }
   virtual double    C(int ind)         { return(Close(ind)-Close(ind+1));}
   
   
                     CSignalSOM();
                     ~CSignalSOM();
                     
   //--- methods of setting adjustable parameters
   void              XMin(int value)              { m_x_min=value;            }
   void              XMax(int value)              { m_x_max=value;            }
   void              YMin(int value)              { m_y_min=value;            }
   void              YMax(int value)              { m_y_max=value;            }
   
   
 //protected:
   //--- method for learning
   bool              SOMLearn();//double &Inputs[]);
   //--- method for weight adjustment
   void              SOMAdjust(int x,int y,string in,double learn_rate,double neighbourhood_influence);
   //--- method for cordinate of best node
   void              SOMBest(string in);
   //--- methods for parsing weights
   void              SOMInput(string in);
   string            SOMProcess(double _o,double _h,double _v,double _l,double _c);
   void              SOMOutput(string out);
   //---
   //void              Debugger(); 
   
 protected:
   //--- other methods 
   string            m_weights_file;
   //---
   bool              m_learnt;
   int               m_learn_iterations;
   int               m_iterations_count;
   double            m_neighbourhood_radius;
   double            m_neighbourhood_influence;
   double            m_initial_learn_rate;
   double            m_learn_rate;
   double            m_time_constant;
   
   int               X,Y;
   double            o,h,v,l,c;
   double            o_old,h_old,v_old,l_old,c_old;
   double            o_new,h_new,v_new,l_new,c_new;
   
   //--- debugging
   public:
   string            grid(int x,int y)      { return GridWeights[x][y];}
   double            o_value()              { return o;                }
   double            h_value()              { return h;                }
   double            v_value()              { return v;                }
   double            l_value()              { return l;                }
   double            c_value()              { return c;                }
  };
//+------------------------------------------------------------------+
//| Constructor CSignalSOM.                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSignalSOM::CSignalSOM()
  {
      m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE+USE_SERIES_TICK_VOLUME;
      
      //--- setting default values for the oscillator parameters
      m_initial_learn_rate=0.7;
      m_learnt=false;
      
      //--- initializing of weights buffer
      m_weights_file = "SOMWeights " + IntegerToString(ArraySize(GridWeights));
      int handle = FileOpen(m_weights_file+".dat",FILE_READ|FILE_TXT|FILE_BIN);
      if(handle!=INVALID_HANDLE)
      {
         printf(__FUNCTION__+" Read array!! ");
         FileReadArray(handle,GridWeights,0,WHOLE_ARRAY);
         FileClose(handle);
      }
      if(FileSize(handle)<=0||handle==INVALID_HANDLE)
      {  
         printf(__FUNCTION__+" unable to read array!! ");
         
         //CREATE RANDOM VALUES FOR TRAINING
         
         /*for(int i=0; i<GRID; i++)
         {
            for(int j=0; j<GRID; j++)
            {
               GridWeights[i][j] = SOMProcess( (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%10000*0.0001)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1
                                                      );
            }
         }*/
         
      }
  }
//+------------------------------------------------------------------+
//| Destructor CSignalSOM.                                    |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSignalSOM::~CSignalSOM()
  {
  
  //BACKUP MAP ARCHIVE
  
   /*m_weights_file = "SOMWeights " + IntegerToString(ArraySize(GridWeights));
   int handle = FileOpen(m_weights_file+".dat",FILE_WRITE|FILE_TXT|FILE_ANSI);
   if(handle!=INVALID_HANDLE)
   {
      FileWriteArray(handle,GridWeights,0,WHOLE_ARRAY);
      FileClose(handle);
   }
   if(handle==INVALID_HANDLE)
   {
      printf(__FUNCTION__+": error writing weights to file");
   }*/
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//| INPUT:  no.                                                      |
//| OUTPUT: true-if settings are correct, false otherwise.           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalSOM::ValidationSettings()
  {
//--- validation settings of additional filters
   if(!CExpertSignal::ValidationSettings()) return(false);
//--- initial security check
   if(!MQL5InfoInteger(MQL5_TESTING))
   {
      if(m_account.Login()!=9044767)//m_symbol.Bank()!="NordFX"
        {
         printf(__FUNCTION__+": please acquire a license by contacting hornsVIEW@gmail.com");
         return(false);
        }
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Checks conditions for buy                                        |
//| INPUT:  None                                                     |
//| OUTPUT: Vote weight from 0 to 100                                |
//| REMARK: None.                                                    |
//+------------------------------------------------------------------+
int CSignalSOM::LongCondition()
  {
   //SOMLearn();
   //return(0);
   
   //printf(__FUNCTION__+" corner weights are : "+GridWeights[0][0]+" , "+GridWeights[0][GRID-1]+" , "+GridWeights[GRID-1][0]+" , "+GridWeights[GRID-1][GRID-1]+" for 0,0; 0,GRID-1; GRID-1,0; and  GRID-1,GRID-1 respectively. ");
   
   return(0);
   
   int idx = StartIndex();
   SOMBest(SOMProcess(O(idx),H(idx),V(idx),L(idx),C(idx)));
//--- check buy conditions for last completed bar
   //datetime timer=TimeCurrent();
   if(X>m_x_max) return (0);
   //datetime X_timer=TimeCurrent();
   //printf(__FUNCTION__+" x max @ "+DoubleToString(X_timer-timer,8));
   if(X<m_x_min) return (0);
   //datetime x_timer=TimeCurrent();
   //printf(__FUNCTION__+" x min @ "+DoubleToString(x_timer-timer,8));
   if(Y>m_y_max) return (0);
   //datetime Y_timer=TimeCurrent();
   //printf(__FUNCTION__+" y max @ "+DoubleToString(Y_timer-timer,8));
   if(Y<m_y_min) return (0);
   //datetime y_timer=TimeCurrent();
   //printf(__FUNCTION__+" y min @ "+DoubleToString(y_timer-timer,8));
      
//--- condition is satisfied
   return(100);
  }
//+------------------------------------------------------------------+
//| Checks conditions for sell                                       |
//| INPUT:  None                                                     |
//| OUTPUT: Vote weight from 0 to 100                                |
//| REMARK: None.                                                    |
//+------------------------------------------------------------------+
int CSignalSOM::ShortCondition()
  {
   //SOMLearn();
   //return(0);
   
   printf(__FUNCTION__+" center weights are : "+GridWeights[GRID/2-1][GRID/2-1]);
   
   
   //printf(__FUNCTION__+" corner weights are : "+GridWeights[0][0]+" , "+GridWeights[0][GRID-1]+" , "+GridWeights[GRID-1][0]+" , "+GridWeights[GRID-1][GRID-1]+" for 0,0; 0,GRID-1; GRID-1,0; and  GRID-1,GRID-1 respectively. ");
   
   return(0);
   
   int x_min,x_max,y_min,y_max;

   x_max=(m_x_max>0.5*GRID)?int(NormalizeDouble(m_x_max-GRID,0)):(m_x_max<0.5*GRID)?int(NormalizeDouble(GRID-m_x_max,0)):m_x_max;

   x_min=(m_x_min>0.5*GRID)?int(NormalizeDouble(m_x_min-GRID,0)):(m_x_min<0.5*GRID)?int(NormalizeDouble(GRID-m_x_min,0)):m_x_min;

   y_max=(m_y_max>0.5*GRID)?int(NormalizeDouble(m_y_max-GRID,0)):(m_y_max<0.5*GRID)?int(NormalizeDouble(GRID-m_y_max,0)):m_y_max;

   y_min=(m_y_min>0.5*GRID)?int(NormalizeDouble(m_y_min-GRID,0)):(m_y_min<0.5*GRID)?int(NormalizeDouble(GRID-m_y_min,0)):m_y_min;
   
   int idx = StartIndex();
   SOMBest(SOMProcess(O(idx),H(idx),V(idx),L(idx),C(idx)));
//--- check sell conditions for last completed bar
   //datetime timer=TimeCurrent();
   if(X>x_max) return (0);
   //datetime X_timer=TimeCurrent();
   //printf(__FUNCTION__+" x max @ "+DoubleToString(X_timer-timer,8));
   if(X<x_min) return (0);
   //datetime x_timer=TimeCurrent();
   //printf(__FUNCTION__+" x min @ "+DoubleToString(x_timer-timer,8));
   if(Y<y_max) return (0);
   //datetime Y_timer=TimeCurrent();
   //printf(__FUNCTION__+" y max @ "+DoubleToString(Y_timer-timer,8));
   if(Y>y_min) return (0);
   //datetime y_timer=TimeCurrent();
   //printf(__FUNCTION__+" y min @ "+DoubleToString(y_timer-timer,8));
      
   
//--- condition is satisfied
   return(100);
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
bool CSignalSOM::SOMLearn()//double &Inputs[])
  {
      //make sure the size of the input vector matches the size of each node's
      //weight vector
      //if(ArraySize(Inputs)!=INPUT) return (false);
  
      //return if the training is complete
      if(m_learnt) return(true);
      
      m_learn_iterations = 1;Bars(Symbol(),PERIOD_W1,TimeCurrent()-12*PeriodSeconds(PERIOD_MN1),TimeCurrent());
      
      int idx = StartIndex(),l_timer=int(TimeCurrent());
      m_iterations_count=0;
      
      //enter the training loop
      for (int l=m_learn_iterations-1; l>=0; l--)
      {
         //chose a vector at random from the training set to be
         //this time-step's input vector
         string learn_input = SOMProcess(O(idx+l),H(idx+l),V(idx+l),L(idx+l),C(idx+l));
         //---
         printf(__FUNCTION__+" input string: "+learn_input+" at time : "+TimeToString(TimeCurrent(),TIME_DATE));
         //---
         //present the vector to each node and determine the BMU
         
         SOMBest(learn_input);
         
         //calculate the width of the neighbourhood for this timestep
         //m_dMapRadius = max(constWindowWidth, constWindowHeight)/2;
         //m_dTimeConstant = m_iNumIterations/log(m_dMapRadius);
         //m_dNeighbourhoodRadius = m_dMapRadius * exp(-(double)m_iIterationCount/m_dTimeConstant);
         //m_iterations_count = 10;//for testing
         m_time_constant = m_learn_iterations/MathExp(GRID/2);
         m_neighbourhood_radius = (GRID/2)*MathExp(-double(m_iterations_count)/m_time_constant);
         //printf(__FUNCTION__+" neighbourhood radius: "+string(m_neighbourhood_radius));
         
         //Now to adjust the weight vector of the BMU and its
         //neighbours

         //For each node calculate the m_neighbourhood_influence (Theta from equation 6 in
         //the tutorial). If it is greater than zero adjust the node's weights
         //accordingly
         //int n_timer=int(TimeCurrent());
         double squared_distance=0.0,squared_width=0.0;
         for (int m=0; m<GRID; m++)
         {
           //calculate the Euclidean distance (squared) to this node from the
           //BMU
            for (int n=0; n<GRID; n++)
            {
               squared_distance = MathPow(X-m,2.0)+MathPow(Y-n,2.0);
               squared_width = MathPow(m_neighbourhood_radius,2.0);
               
               //if within the neighbourhood adjust its weights
               if (squared_distance < squared_width)
               {
                  //calculate by how much its weights are adjusted
                  m_neighbourhood_influence = MathExp(-(squared_distance)/(2*squared_width));
                  //printf(__FUNCTION__+" neighbourhood influence: "+string(m_neighbourhood_influence));
                  
                  SOMAdjust(m,n,learn_input,m_learn_rate,m_neighbourhood_influence);
               }
               //next node...
            }
            printf(__FUNCTION__+" adjusted weights... ");
            //reduce the learning rate
            m_learn_rate = m_initial_learn_rate*MathExp(-double(m_iterations_count)/m_learn_iterations);
            m_iterations_count++;
            printf(__FUNCTION__+" adjusted learning rate to... "+string(m_learn_rate));
         }
         //int n_lag=int(TimeCurrent())-n_timer;
         //printf(__FUNCTION__+" m & n for loop in "+string(n_lag)+" seconds. ");
      }
      //int l_lag=int(TimeCurrent())-l_timer;
      //printf(__FUNCTION__+" l for loop in "+string(l_lag)+" seconds. ");
      
      
      //else
      //{
      //  m_learnt = true;
      //}
   return(true);
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
void CSignalSOM::SOMAdjust(int x,int y,string in,double learn_rate,double neighbourhood_influence)
  {
      SOMInput(in);
      
      string out = GridWeights[x][y];
      
      SOMOutput(out);
      
      o_new = o_old+neighbourhood_influence*learn_rate*(o-o_old);
      h_new = o_old+neighbourhood_influence*learn_rate*(h-h_old);
      v_new = o_old+neighbourhood_influence*learn_rate*(v-v_old);
      l_new = o_old+neighbourhood_influence*learn_rate*(l-l_old);
      c_new = o_old+neighbourhood_influence*learn_rate*(c-c_old);
      
      GridWeights[x][y] =  SOMProcess(o_new,h_new,v_new,l_new,c_new);
      //printf(__FUNCTION__+" adjusted weights for: "+string(x)+" , "+string(y));
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
void CSignalSOM::SOMBest(string in)
  {
      SOMInput(in);   
      
      string out;
      X=int(NormalizeDouble(0.5*GRID,0));
      Y=int(NormalizeDouble(0.5*GRID,0));
      double distance=MathSqrt(2*MathPow(GRID,2.0)), measure;
      for (int m=0; m<GRID; m++)
      {
         for (int n=0; n<GRID; n++)
         {
            out = GridWeights[m][n];
            SOMOutput(out);
            
            measure = 0.0;
            measure = MathSqrt(MathPow(o-o_old,2.0)+MathPow(h-h_old,2.0)+MathPow(v-v_old,2.0)+MathPow(l-l_old,2.0)+MathPow(c-c_old,2.0));
            if(distance>measure)
            {
               distance = measure;
               X = m;
               Y = n;
            }
         }
      }
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
void CSignalSOM::SOMInput(string in)
  {
      o = StringToDouble(StringSubstr(in,StringFind(in,"!",0)+1,StringFind(in,"@",0)-1));
      h = StringToDouble(StringSubstr(in,StringFind(in,"@",0)+1,StringFind(in,"#",0)-StringFind(in,"@",0)));
      v = StringToDouble(StringSubstr(in,StringFind(in,"#",0)+1,StringFind(in,"$",0)-StringFind(in,"#",0)));
      l = StringToDouble(StringSubstr(in,StringFind(in,"$",0)+1,StringFind(in,"%",0)-StringFind(in,"$",0)));
      c = StringToDouble(StringSubstr(in,StringFind(in,"%",0)+1,StringFind(in,"^",0)-StringFind(in,"%",0)));
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
string CSignalSOM::SOMProcess(double _o,double _h,double _v,double _l,double _c)
  {
      string processed =   "!"+DoubleToString(_o,5)+
                           "@"+DoubleToString(_h,5)+
                           "#"+DoubleToString(_v,5)+
                           "$"+DoubleToString(_l,5)+
                           "%"+DoubleToString(_c,5)+
                           "^";
      return(processed);
  }
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
void CSignalSOM::SOMOutput(string out)
  {
      o_old = StringToDouble(StringSubstr(out,StringFind(out,"!",0)+1,StringFind(out,"@",0)-1));
      h_old = StringToDouble(StringSubstr(out,StringFind(out,"@",0)+1,StringFind(out,"#",0)-StringFind(out,"@",0)));
      v_old = StringToDouble(StringSubstr(out,StringFind(out,"#",0)+1,StringFind(out,"$",0)-StringFind(out,"#",0)));
      l_old = StringToDouble(StringSubstr(out,StringFind(out,"$",0)+1,StringFind(out,"%",0)-StringFind(out,"$",0)));
      c_old = StringToDouble(StringSubstr(out,StringFind(out,"%",0)+1,StringFind(out,"^",0)-StringFind(out,"%",0)));
  }
  
//+------------------------------------------------------------------+
//| Checks                                                           |
//| INPUT:                                                           |
//| OUTPUT:                                                          |
//| REMARK:                                                          |
//+------------------------------------------------------------------+
/*void CSignalSOM::Debugger(void)
  {
      m_weights_file = "SOMWeights " + IntegerToString(ArraySize(GridWeights));
      int handle = FileOpen(m_weights_file+".dat",FILE_READ|FILE_TXT|FILE_ANSI);
      if(handle!=INVALID_HANDLE)
      {
         FileReadArray(handle,GridWeights,0,WHOLE_ARRAY);
         printf(__FUNCTION__+" weights written to grid ");
         FileClose(handle);
      }
      //printf(__FUNCTION__+" handle is invalid ");
      
      
      if(handle==INVALID_HANDLE)
      {  
         for(int i=0; i<GRID; i++)
         {
            for(int j=0; j<GRID; j++)
            {
               GridWeights[i][j] = SOMProcess( (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1,
                                                      (MathRand()%20000*0.0001-1.0)*0.1);
            }
         }
      }
  }*/
//+------------------------------------------------------------------+
